{% extends "python.j2" %}
{% block imports %}
from codecs import open
from setuptools import setup, find_packages
{% endblock %}

{% block code %}

# HISTORY.rst entry.
VERSION = '1.0.0b1'

# The full list of classifiers is available at
# https://pypi.python.org/pypi?%3Aaction=list_classifiers
CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Intended Audience :: System Administrators',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'License :: OSI Approved :: MIT License',
]

DEPENDENCIES = []

with open('README.md', 'r', encoding='utf-8') as f:
    README = f.read()
with open('HISTORY.rst', 'r', encoding='utf-8') as f:
    HISTORY = f.read()

setup(
    name='{{ mod_name }}',
    version=VERSION,
    description='Microsoft Azure Command-Line Tools {{ mod_name|camel_case }} Extension.',
    long_description=README + '\n\n' + HISTORY,
    license='MIT',
    author='Microsoft Corporation',
    author_email='azpycli@microsoft.com',
    url='https://github.com/Azure/azure-cli-extensions/tree/main/src/{{ mod_name }}',
    classifiers=CLASSIFIERS,
    packages=find_packages(exclude=["tests"]),
    package_data={'azext_{{ mod_name|snake_case }}': ['azext_metadata.json']},
    install_requires=DEPENDENCIES
)
{% endblock %}
